@@ -29,6 +29,12 @@
 
 #include "desktop-clock-widget.h"
 
+#define HD_APP_MGR_DBUS_NAME "com.nokia.HildonDesktop.AppMgr"
+#define HD_APP_MGR_DBUS_PATH "/com/nokia/HildonDesktop/AppMgr"
+#define HD_APP_MGR_DBUS_IFACE "com.nokia.HildonDesktop.AppMgr"
+#define HD_APP_MGR_LAUNCH_APPLICATION "LaunchApplication"
+#define WORLD_CLOCK_APP_NAME "worldclock"
+
 #define DESKTOP_CLOCK_WIDGET_HOME_PLUGIN_SETTINGS_FILE "config"
 #define DESKTOP_CLOCK_WIDGET_HOME_PLUGIN_SETTINGS_FOLDER ".desktop_clock_widget"
 #define DESKTOP_CLOCK_WIDGET_CONFIG_KEY_ZONE_ID "zone_id"
@@ -464,10 +470,26 @@ desktop_clock_widget_write_settings(DesktopClockWidgetHomePlugin* desktop_plugin
 			 desktop_plugin->priv->iD, 
 			 DESKTOP_CLOCK_WIDGET_CONFIG_KEY_ZONE_ID, 
 			 desktop_plugin->priv->city_info_id);
-  g_key_file_set_string(key_file, 
-			 desktop_plugin->priv->iD, 
-			 DESKTOP_CLOCK_WIDGET_CONFIG_KEY_CUSTOM_THEME_NAME,			 
-			 desktop_plugin->priv->custom_theme_name);  
+  if(desktop_plugin->priv->custom_theme_name!=NULL)
+  {
+    g_key_file_set_string(key_file, 
+			  desktop_plugin->priv->iD, 
+			  DESKTOP_CLOCK_WIDGET_CONFIG_KEY_CUSTOM_THEME_NAME,			 
+			  desktop_plugin->priv->custom_theme_name);
+  }
+  else
+  {
+    if(g_key_file_has_key(key_file, 
+			  desktop_plugin->priv->iD, 
+			  DESKTOP_CLOCK_WIDGET_CONFIG_KEY_CUSTOM_THEME_NAME, 
+			  NULL))
+    {
+      g_key_file_remove_key(key_file,
+			    desktop_plugin->priv->iD, 
+			    DESKTOP_CLOCK_WIDGET_CONFIG_KEY_CUSTOM_THEME_NAME,
+			    NULL);
+    }
+  }
   file_data = g_key_file_to_data (key_file, &size, NULL);
   g_file_set_contents(file_name, file_data, size, NULL);
   
@@ -629,7 +651,6 @@ desktop_clock_widget_select_theme(GtkButton* button, gpointer data)
   GtkWidget* parent = gtk_widget_get_toplevel(GTK_WIDGET(button));
   DesktopClockSVGTheme* theme = (DesktopClockSVGTheme*)data;
   parent = gtk_widget_get_parent(parent);
-
   dialog = hildon_file_chooser_dialog_new(GTK_WINDOW(parent),
 					  GTK_FILE_CHOOSER_ACTION_SELECT_FOLDER);
   gtk_widget_show_all(GTK_WIDGET(dialog));
@@ -940,6 +961,7 @@ desktop_clock_widget_show_settings_dialog(GtkWidget* widget, gpointer data)
     int new_width = gtk_range_get_value(GTK_RANGE(clock_width_slider));
     int new_height = gtk_range_get_value(GTK_RANGE(clock_height_slider));
     gboolean use_stock_theme = hildon_check_button_get_active(HILDON_CHECK_BUTTON(stock_theme_checkbutton));
+    
     gboolean reread_theme = 
       new_width!=desktop_plugin->priv->clock_width ||
       new_height!=desktop_plugin->priv->clock_height ||
@@ -964,7 +986,7 @@ desktop_clock_widget_show_settings_dialog(GtkWidget* widget, gpointer data)
       desktop_plugin->priv->city_info = cityinfo_from_id(id);
       desktop_plugin->priv->city_info_id = id;
     }
-
+    
     if(!desktop_plugin->priv->use_stock_theme && theme->path!=NULL)
     {
       desktop_clock_widget_save_custom_theme(theme->path, desktop_plugin);
@@ -1053,10 +1075,38 @@ desktop_clock_widget_init_default_timezone(DesktopClockWidgetHomePlugin* desktop
 static GtkWidget*
 build_ui(void)
 {
-  GtkWidget* contents = gtk_vbox_new(FALSE, 0);
+  GtkWidget* contents = gtk_event_box_new();
+  gtk_event_box_set_visible_window(GTK_EVENT_BOX(contents), FALSE);
+  gtk_container_set_border_width(GTK_CONTAINER(contents), 0);
+  gtk_widget_show(contents);
   return contents;
 }
 
+static gboolean
+desktop_clock_widget_open_clock_app(GtkWidget* widget, GdkEventButton* event, DesktopClockWidgetHomePlugin* desktop_plugin)
+{
+  DBusGConnection* dbus_conn = hd_home_plugin_item_get_dbus_g_connection(&desktop_plugin->hitem, 
+									 DBUS_BUS_SESSION, 
+									 NULL);
+  DBusGProxy* dbus_proxy = dbus_g_proxy_new_for_name(dbus_conn,
+						     HD_APP_MGR_DBUS_NAME,
+						     HD_APP_MGR_DBUS_PATH,
+						     HD_APP_MGR_DBUS_IFACE);
+  if(dbus_proxy)
+  {
+    
+    dbus_g_proxy_call_no_reply(dbus_proxy,
+			       HD_APP_MGR_LAUNCH_APPLICATION,
+			       G_TYPE_STRING,
+			       WORLD_CLOCK_APP_NAME,
+			       G_TYPE_INVALID,
+			       G_TYPE_INVALID);
+    
+    g_object_unref(dbus_proxy);
+  }
+  return FALSE;
+}
+
 static gboolean
 desktop_clock_widget_expose(GtkWidget* widget, GdkEventExpose *event)
 {
@@ -1234,7 +1284,6 @@ desktop_clock_widget_finalize(GObject* object)
   G_OBJECT_CLASS(desktop_clock_widget_home_plugin_parent_class)->finalize(object);
 }
 
-
 static void
 desktop_clock_widget_home_plugin_init(DesktopClockWidgetHomePlugin* desktop_plugin)
 {
@@ -1265,6 +1314,7 @@ desktop_clock_widget_home_plugin_init(DesktopClockWidgetHomePlugin* desktop_plug
   hd_home_plugin_item_set_settings (HD_HOME_PLUGIN_ITEM (desktop_plugin), TRUE);
   g_signal_connect(desktop_plugin, "show-settings", G_CALLBACK(desktop_clock_widget_show_settings_dialog), NULL);
   g_signal_connect(desktop_plugin, "notify::is-on-current-desktop", G_CALLBACK(desktop_clock_widget_on_current_desktop), NULL);
+  g_signal_connect(GTK_CONTAINER(contents), "button-release-event", G_CALLBACK(desktop_clock_widget_open_clock_app), desktop_plugin);
   gtk_container_add(GTK_CONTAINER(desktop_plugin), contents);
 }
 
